import type { Identifier } from '@/parser';

export type UnionDenotableType =
  | Array<DenotableType | DenotableFunctionSignature>
  | DenotableType
  | DenotableFunctionSignature;

export type DenotableFunctionSignature = {
  arguments: Array<UnionDenotableType>;
  return: UnionDenotableType;
};

export type DenotableFunction = {
  signatures: Array<DenotableFunctionSignature>;
  body: Function;
};

export type DenotableType =
  | 'null'
  | 'int'
  | 'real'
  | 'string'
  | 'bytearray'
  | 'function'
  | 'reference';

export type DenotableValue =
  | null
  | number
  | string
  | Uint8Array
  | DenotableFunction
  | Identifier;

export type Denotable = {
  type: DenotableType;
  value: DenotableValue;
};

export const functionSignaturesEqual = (
  a: DenotableFunctionSignature,
  b: DenotableFunctionSignature,
): boolean => {
  if (a.arguments.length !== b.arguments.length) {
    return false;
  }

  if (typeof a.return !== typeof b.return) {
    return false;
  }

  if (
    typeof a.return === 'object' &&
    typeof b.return === 'object' &&
    'return' in a.return &&
    'return' in b.return
  ) {
    return functionSignaturesEqual(a.return, b.return);
  }
};

export const matchSignature = (
  args: Array<DenotableFunction | DenotableType>,
  signatures: Array<DenotableFunctionSignature>,
): DenotableFunctionSignature | undefined => {
  return signatures.find(signature => {
    if (args.length !== signature.arguments.length) {
      return false;
    }
  });
};
